%--------------------------------------------------------
%%%%%%%%                UMD Hacks               %%%%%%%%%
%--------------------------------------------------------
% 2018-12-12: payerle
%   getCommonSubmitArgs(): add UMD parameters for
%	WallTime: max walltime for job, in minutes
%	MemUsage: memory (MB) required per CPU (e.g. --mem-per-cpu flag in Slurm)
%	ProjectName: name of Slurm allocation account to charge
%	QueueName: name of Slurm partition to submit to
%	UseGpu, GpusPerNode: boolean, integer
%		If UseGpu, will add --gres=gpu or --gres=gpu:$GpusPerNode
%	EmailAddress: if set, email this address on job status changes
%	Reservation: if set, use specified Slurm reservation
%	??? DataParallelism --- never implemented in old, not in
%		mathwork docs (https://www.mathworks.com/matlabcentral/fileexchange/32441-clusterinfo)
%	UserDefinedOptions: user can set a string of flags to give to Slurm sbatch
%
% 	Old version (getAdditionalSubmitArguments) also added 
%		--ntasks=N and --licenses=mdcs:N
%		But we no longer know number of tasks, move this to
%		communicatingSubmitFunction.m
%--------------------------------------------------------

function commonSubmitArgs = getCommonSubmitArgs(cluster)
% Get any additional submit arguments for the Slurm sbatch command
% that are common to both independent and communicating jobs.

% Copyright 2016-2017 The MathWorks, Inc.

commonSubmitArgs = '';

% Append any arguments provided by the AdditionalSubmitArgs field of cluster.AdditionalProperties.
if isprop(cluster.AdditionalProperties, 'AdditionalSubmitArgs')
    extraArgs = cluster.AdditionalProperties.AdditionalSubmitArgs;
    if ~isempty(extraArgs) && ischar(extraArgs)
        commonSubmitArgs = strtrim([commonSubmitArgs, ' ', extraArgs]);
    end
end

% You may wish to support further cluster.AdditionalProperties fields here
% and modify the submission command arguments accordingly.
%
%

%Walltime
walltime = ClusterInfo.getWallTime();
if isempty(walltime) == false
	commonSubmitArgs = [ commonSubmitArgs ' -t ' walltime ];
end

%MemUsage
%We only support mem-per-cpu; use UserDefinedOptions to set --mem (mem per node)
memusage = ClusterInfo.getMemUsage();
if isempty(memusage) == false
	commonSubmitArgs = [ commonSubmitArgs ' --mem-per-cpu=' memusage ];
end

%ProjectName: specify slurm allocation account to charge against
projname = ClusterInfo.getProjectName();
if isempty(projname) == false
	commonSubmitArgs = [ commonSubmitArgs ' -A ' projname ];
end

%QueueName: specify partition to use
qname = ClusterInfo.getQueueName();
if isempty(qname) == false
	commonSubmitArgs = [ commonSubmitArgs ' -p ' qname ];
end

%UseGpu and GpuPerNode
usegpu = ClusterInfo.getUseGpu();
if usegpu == true
	%Want to use GPUs, did they specify number of GPUs/node?
	numgpus = ClusterInfo.getGpusPerNode();
	if isempty(numgpus) == true
		%They did not specify a number
		%Just do --gres=gpu flag
		cntStr = '';
	else
		%They specified a number
		%Append to --gres=gpu: flag
		cntStr = sprintf( ':%d', numgpus );
	end
	commonSubmitArgs = sprintf( '%s --gres=gpu%s', commonSubmitArgs,  qname);
end

%EmailAddress: email user at this address on changes to job status
emailaddr = ClusterInfo.getEmailAddress();
if isempty(emailaddr) == false
	%Currently email on ALL status changes (or none)
	commonSubmitArgs = [ commonSubmitArgs ' --mail-type=ALL --mail-user ' emailaddr ];
end
	
%Reservation: use specified Slurm reservation if given
reserv = ClusterInfo.getReservation();
if isempty(reserv) == false
	commonSubmitArgs = [ commonSubmitArgs ' --reservation ' reserv ];
end

%UserDefinedOptions: allow users to add any other options they need
%	Just provide a string here which will be appended to other args
userdef = ClusterInfo.getUserDefinedOptions();
if isempty(userdef) == false
	commonSubmitArgs = [ commonSubmitArgs ' ' reserv ];
end

%Old code had a DataParallelism section, but left in TODO mode
%Does not appear in current ClusterInfo docs at mathworks site
%Leaving commented out for now
% datapar = ClusterInfo.getDataParallelism();
% if isempty(datapar) == false
%	% TODO
% end


	
