% This is a major rewrite of code originally provided by MathWorks
%
% Configure MATLAB for users running remotely to the cluster.
%   1. Imports partial profile
%   2. Determines and creates (if needed) local job storage location
%   3. Updates profile and sets it as the default
%
% Handles for both pre-2017a or 2017a+ profile formats.

function configCluster
%-----------     Identify ourselves
fprintf('configCluster: version 2.1.0\n\n');

% University of Maryland
site = 'umd';
% The location of MATLAB as compared to the cluster
type = 'remote';
% The version of MATLAB being supported
myMatlabRelease = version('-release');
release = ['R' myMatlabRelease ];
% The directory where the current script is located
mymdir = fileparts(mfilename('fullpath'));

% Determine which profile format should be using
% 	2016b: format used by 2014a (maybe earlier???) through 2016b
%	2017a: format used by 2017a and later
profileFormat = '';
versionCutoff = '2017a';
tmpVersions = { myMatlabRelease; versionCutoff };
[ ~, idx ]  = sort(tmpVersions);
tmpVersion = tmpVersions(idx(1));
if strcmp(tmpVersion, versionCutoff )
	% Version of matlab running >= versionCutoff
	profileFormat = '2017a';
else
	% Version of matlab running is older than versionCutoff
	profileFormat = '2016b';
end

%----------	Select the profile to use
% The directory where we should look for profiles:
%	for now assume profiles in the same dir as configCluster script
profile_dir = mymdir;

% Look for all profiles (*.settings files) matching our MATLAB version
pfile_pattern = fullfile(profile_dir, [ '*r' myMatlabRelease '*.settings' ]);
pfiles = dir(pfile_pattern);
if isempty(pfiles)==true
	warning('No profiles matching matlab version found, trying all profiles.  Might not work.')
	%# No files matching version found, try a bit broader
	pfile_pattern = fullfile(profile_dir, '*.settings');
	pfiles = dir(pfile_pattern);
end
if isempty(pfiles)==true
	error('Failed to find profiles.  Contact your System Administrator.')
elseif length(pfiles)>1
	% Multiple pfiles found, ask user to pick one
	[ pfile, pfullpath ]  = lExtractPfile(pfiles);
else
	% Only single pfile found, use it
	pfile = pfiles.name;
	pfullpath = fullfile(profile_dir, pfiles.name);
	fprintf('INFO: Using profile: %s\n', pfullpath);
end

%----------	Compute/query user for various config settings
cluster = '';
uname = '';
idfile = '';
idhaspass = false;

% Try to determine cluster name from settings file
cluster = strtok(pfile,'_');
if strcmp(pfile, cluster) == 1
	% cluster == pfile, i.e. no _ in pfile
	warning('Unable to derive cluster name from profile file name.');
	cluster = input( ['Name of cluster for profile ' pfile ': '], 's');
	if isempty(cluster) == true
		error(['Failed to configure cluster: no cluster name given']);
	end
end

% Set cluster specifics
%	cluster: name of cluster
%	submit_host: hostname of nodes should ssh to in order to submit jobs
%	cluster_jobstorage_root: root of remote job storage location on cluster
cluster = lower(cluster);
submit_host = '';
cluster_jobstorage_root = '';

switch cluster
    case { 'dt2', 'deepthought2', 'dt', 'deepthought' }
	cluster = 'deepthought2';
	submit_host = 'login-3.deepthought2.umd.edu';
	cluster_jobstorage_root = '/lustre';

    case { 'jn', 'juggernaut' }
	cluster = 'juggernaut';
	submit_host = 'login.juggernaut.umd.edu';
	cluster_jobstorage_root = '/lustre/jn10';

    case { 'bswift' }
	cluster = 'bswift';
	submit_host = 'login.bswift.umd.edu';
	cluster_jobstorage_root = '/data/bswift-1';

    otherwise
	error(['Failed to configure cluster: unrecognized cluster ' cluster '; expecting deepthought2 or juggernaut or bswift']);
end

switch profileFormat
    case '2016b'
	%Old version ( <2017a)

	% We require the integration scripts to be in package profiles.$site.$cluster
	% underneath directory containing configCluster
	% But we need to generate the packageName and the directory path
	profilePackage_comps = { 'profiles', site, cluster };
	slurmint_dir = profile_dir;
	profilePackageName = '';

	for idx = 1:length(profilePackage_comps)
		tmpcomp = char(profilePackage_comps(idx));
		% Generate profile Namespace
		if isempty(profilePackageName)
			profilePackageName = tmpcomp;
		else
			profilePackageName = sprintf('%s.%s', profilePackageName, tmpcomp);
		end

		%Generate path to integration scripts
		slurmint_dir = fullfile(slurmint_dir, [ '+' tmpcomp ] );
	end

	wanted_files = { ...
		'communicatingJobWrapper.sh-gige', ...
		'communicatingSubmitFcn.m', ...
		'createSubmitScript.m', ...
		'deleteJobFcn.m', ...
		'extractJobId.m', ...
		'getAdditionalSubmitArguments.m', ...
		'getJobStateFcn.m', ...
		'getRemoteConnection.m', ...
		'getSubmitString.m', ...
		'independentJobWrapper.sh', ...
		'independentSubmitFcn.m', ...
	};

    case '2017a'
	%New Generic format from 2017a onwards

	%Location of Slurm integration scripts
	slurmint_dir = fullfile( mymdir, 'UMD-Slurm-Integration-Scripts' ); 
	fprintf('\nI need to know where the Slurm integration scripts are located.\n');
	fprintf('That is the directory with communicatingSubmitFcn.m, independentSubmitFcn.m\n');
	fprintf('and similar files that were part of matlab_umd_remote.zip or tarball are located.\n');
	fprintf('I am going to assume\n	"%s"\n unless you tell me something different.\n', slurmint_dir);
	fprintf('That is where it would be if you did not move it after\n');
	fprintf('unzipping/untarring.  Just hit return to use the default,\n');
	fprintf('or enter the new location if you moved it.\n');
	tmp = input('Enter the location of Slurm integration scripts (or hit return for default): ', 's');
	if ~isempty(tmp)
		% User specified a different directory
		slurmint_dir = tmp;
	end

	wanted_files = { ...
		'cancelJobFcn.m', ...
		'communicatingJobWrapper.sh', ...
		'communicatingJobWrapperSmpd.sh', ...
		'communicatingSubmitFcn.m', ...
		'createSubmitScript.m', ...
		'deleteJobFcn.m', ...
		'extractJobId.m', ...
		'getCommonSubmitArgs.m', ...
		'getJobStateFcn.m', ...
		'getRemoteConnection.m', ...
		'getSubmitString.m', ...
		'independentJobWrapper.sh', ...
		'independentSubmitFcn.m', ...
	};

    otherwise
	error(sprintf('Failed to configure cluster: unrecognized profile format "%s"\n', profileFormat));
end

% Validate that requisite files in slurmint_dir
file_pattern = fullfile(slurmint_dir, '*' );
% dir returns array of structs
files_found = dir(file_pattern);
% We need to extra the name field from each
fnames_found = arrayfun(@(x) x.name, files_found, 'uniformoutput', 0);
for idx = 1:length(wanted_files)
	wanted = char(wanted_files(idx));
	tmp = find(strcmp(fnames_found, wanted));
	if isempty(tmp)
		error(sprintf('Failed to configure cluster: cannot find file %s in Slurm integration scripts dir %s\n', wanted, slurmint_dir));
	end
end
% If reached here, found all integration scripts

% Get username on the cluster
uname = input(['\n Enter your username on the ' cluster ' cluster: '],'s');
if isempty(uname)
    error(['Failed to configure cluster: ' cluster ': no username given' ])
end

if ~isempty(uname)
	% User name was given
	% Ask for preferences re AuthMode, PrivateKeyFile, PrivateKeyFileHasPassPhrase

	% Question re AuthMode
	authmode = '';
	fprintf('\nHow do you wish to authenticate as %s to the %s cluster?\n', uname, cluster);
	fprintf('Valid options are: \n');
	fprintf('	ask : you will be asked in every MATLAB session.\n');
	fprintf('	password : password authentication.  You will be prompted for password each session.\n');
	fprintf('	identity : using RSA identity file.\n');
        while ~(strcmp(authmode, 'password') | strcmp(authmode, 'identityfile') | strcmp(authmode, 'ask') )
		authmode = input('Enter your authentication preference: ', 's');
		if isempty(authmode)
			error([ 'Failed to configure cluster ' cluster ': no authentication mode given.' ]);
		end
		authmode = lower(authmode);
		switch authmode
		    case 'ask'
			authmode = 'ask';
		    case { 'password', 'pw', 'pwd', 'pass', 'passwd' }
			authmode = 'password';
		    case { 'rsa', 'ssh', 'pubkey', 'id', 'idfile', 'identity', 'identityfile' }
			authmode = 'identityfile';
		    otherwise
			fprintf('Invalid response "%s"; please enter "ask", "password", or "identityfile"\n', authmode);
		end
	end

	if strcmp(authmode, 'identityfile' )
		%Opted for RSA identityfile
		%Give them the option of specifying file and excrypted status
		idfile = '';
		fprintf('You can specify the path to the identity file to use for authentication below.\n');
		fprintf('If you leave it blank, you will be prompted for it each MATLAB session.\n');
		idfile = input([ 'Enter path of identity file to use for authentication to ' cluster ': '], 's');
		if isempty(idfile)
			fprintf('INFO: You will be prompted for identity file each MATLAB session.\n');
		else
			tmp = '';
                        while ~ismember(tmp, {'yes', 'no'}) 
				tmp = input(['Does the identity file ' idfile ' require a passphrase (y/n)?'], 's');
				if isempty(tmp)
					error(['Failed to configure cluster ' cluster ': user aborted question re passphrase for identity file']);
				end
				tmp = lower(tmp);
				switch tmp
				    case { 'yes', 'y', 'true', 't' }
					tmp = 'yes';
					idhaspass = true;
				    case { 'no', 'n', 'false', 'f' }
					tmp = 'no';
					idhaspass = false;
				    otherwise
					fprintf('\nPlease enter "yes" or "no"\n');
				end %switch
			end %while
		end %if isempty(idfile)
	end %if strcmp(authmode, 'identityfile')
end
	
 fprintf('uname=%s\n', uname);
 fprintf('authmode=%s\n', authmode);
 fprintf('idfile=%s\n', idfile);
 fprintf('idhaspass=%s\n', mat2str(idhaspass) );

% Create the user's Job Storage Location folder (local)
local_jobdir = '';
rootd = fullfile(tempdir,uname);
local_jobdir = fullfile(rootd,'MdcsDataLocation',site,cluster,release,type);
if exist(local_jobdir,'dir')==false
    [status,err,eid] = mkdir(local_jobdir);
    if status==false
        error(['Failed to configure cluster ' cluster ': Error creating local job storage location ' local_jobdir ': eid=' eid ' err= ' err]);
    end
end

% Remote Job Storage Location
% Manually insert '/' as remote is unix; don't use fullfile as that 
% uses local directory separator
rootd = '';
switch cluster
 case { 'deepthought2', 'juggernaut', 'bswift' }
	rootd = [ cluster_jobstorage_root '/' uname ];
    otherwise
	error(['Failed to configure cluster ' cluster ': Unable to set rootd for remote job storage location']);
end
rjsl = [ rootd '/MdcsDataLocation/' site '/' cluster '/' release '/' type];

%%2019a started using Intel MPI based MPI by default, which does not work on RH6 (libudev too old)
%%See e.g. https://www.mathworks.com/matlabcentral/answers/460733-why-do-i-see-the-error-mpiexec-libudev-so-1-not-found-in-the-log-file-when-trying-to-run-valid
%So for Deepthought2, always set AdditionalProperty UseSmpd to true, but 
%should be false on Juggernaut
% Default to true
use_smpd = true;
switch cluster
    case { 'deepthought2' }
%%   use_smpd = true       
       use_smpd = false;
    case { 'bswift' }
       use_smpd = true
    case { 'juggernaut' }
	% non-Smpd scripts not working on juggernaut either, wierd issue, not sure what
	use_smpd = false;
	%% use_smpd = true;
	fprintf('[INFO] using SMPD on juggernaut\n');
    otherwise
	use_smpd = true;
end

%-----------     Update the actual profile
% From the settings file, derive the name of the Profile
[~,profile_name] = fileparts(pfile);

% Delete the old profile (if it exists)
%   Set default profile to 'local' temporarily just to avoid warnings
%   Otherwise we will get CollapsedDefaultProfileNoLonger Exists error
%   ('...which is not the name of an existing profile...') if profile to delete is default
parallel.defaultClusterProfile('local');
%   Get names of all profiles (force to lowercase, convert spaces to underscores)
all_profiles = lower(strrep(parallel.clusterProfiles(), ' ', '_'));
idx = strcmp(all_profiles, profile_name);
ps = parallel.Settings;
ps.Profiles(find(idx)).delete;

% IMport the settings file
pname = parallel.importProfile(pfullpath);

% Get a handle to the profile
profile = parcluster(pname);
profile.JobStorageLocation = local_jobdir;

switch profileFormat
    case '2016b'
	%Old, pre2017 format

	%Set IndependentSubmitFcn/CommunicatingSubmitFcn
	tmpFname = [ profilePackageName '.independentSubmitFcn' ];
	tmpFunc = str2func(tmpFname);
	profile.IndependentSubmitFcn = { tmpFunc, submit_host, rjsl };
	tmpFname = [ profilePackageName '.communicatingSubmitFcn' ];
	tmpFunc = str2func(tmpFname);
	profile.CommunicatingSubmitFcn = { tmpFunc, submit_host, rjsl };

	%Set misc job functions
	tmpFname = [ profilePackageName '.getJobStateFcn' ];
	tmpFunc = str2func(tmpFname);
	profile.GetJobStateFcn = { tmpFunc };

	tmpFname = [ profilePackageName '.deleteJobFcn' ];
	tmpFunc = str2func(tmpFname);
	profile.DeleteJobFcn = { tmpFunc };

	tmpFname = [ profilePackageName '.cancelJobFcn' ];
	tmpFunc = str2func(tmpFname);
	profile.CancelJobFcn = { tmpFunc };

    case '2017a'
	profile.IntegrationScriptsLocation = slurmint_dir;
	profile.AdditionalProperties.ClusterHost = submit_host;
	profile.AdditionalProperties.RemoteJobStorageLocation = rjsl;
	profile.AdditionalProperties.UseSmpd = use_smpd;
        if cluster == "deepthought2"
	  profile.AdditionalProperties.AdditionalSubmitArgs = ' --constraint=rhel8 '
	end
    otherwise
	error(sprintf('Failed to configure cluster: unrecognized profile format "%s"\n', profileFormat));
end


fprintf('\nInfo: saving profile "%s"\n', pname);
profile.saveProfile;

% Set as default profile
parallel.defaultClusterProfile(pname);

%-----------     Update the actual profile
% Since we've imported a new profile, in all likelihood, any of the current
% ClusterInfo settings are void
fprintf('Resetting ClusterInfo settings.\n');
ClusterInfo.clear;
ClusterInfo.setUserNameOnCluster(uname);
ClusterInfo.setAuthMode(authmode);
ClusterInfo.setPrivateKeyFile(idfile);
ClusterInfo.setPrivateKeyFileHasPassPhrase(idhaspass);

lNotifyUserOfCluster(cluster, pname)

% % Validate if you want to
% profiles = parallel.clusterProfiles();
% idx = strcmp(profiles,profile);
% ps = parallel.Settings;
% ps.Profiles(idx).validate

end

% This function takes a list of possible files (structured array as 
% returned by dir() function) and asks user to select % one.  
% Returns both the base name and the full path
function [ pfile, pfullpath ] = lExtractPfile(pl)

len = length(pl);
for pidx = 1:len
    name = pl(pidx).name;
    names{pidx,1} = strtok(name,'.'); %payerle: include all but extension
end

selected = false;
while selected==false
    for pidx = 1:len
        fprintf('\t[%d] %s\n',pidx,names{pidx});
    end
    idx = input(sprintf('Select a cluster [1-%d]: ',len));
    selected = idx>=1 && idx<=len;
end
pfile = pl(idx).name;
pfullpath = fullfile( pl(idx).folder, pl(idx).name );

end


function lNotifyUserOfCluster(cluster, pname)

fprintf('Successfully configured profile %s to send jobs to the %s cluster\n', pname, cluster);
fprintf('\n');
fprintf('You will likely need to specify some additional parameters for your job\n');
fprintf('in the ClusterInfo structure, e.g. WallTime, MemUsage, and maybe UseGpu and GpusPerNode.\n');
fprintf('See https://hpcc.umd.edu/help/software/matlab.html#usingMDCS for more information\n.');
fprintf('\n');
fprintf('To set parameter "Foo", you would need to use "ClusterInfo.setFoo(newvalue)"\n');
fprintf('E.g., to set WallTime to 60 minutes: "ClusterInfo.setWallTime(''60'')" \n');
fprintf('\n');
fprintf('To see all parameters, use "ClusterInfo.state".  You can also rerun this configCluster\n');
fprintf('script to reset the authentication preferences, etc.\n');
fprintf('\n');

end
